package envoy.data;

import java.io.Serializable;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;

/**
 * Contains a {@link User}'s login information.<br>
 * <br>
 * Project: <strong>envoy-common</strong><br>
 * File: <strong>LoginCredentials.java</strong><br>
 * Created: <strong>29.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Common v0.2-alpha
 */
public class LoginCredentials implements Serializable {

	private final String	identifier;
	private final long		id;
	private final byte[]	passwordHash;
	private final boolean	registration;

	private static final long serialVersionUID = -7395245059059523314L;

	/**
	 * Creates an instance of {@link LoginCredentials} for a new {@link User}.
	 *
	 * @param identifier the identifier of the user
	 * @param password   the password of the user (will be converted to a hash)
	 * @throws NoSuchAlgorithmException if the algorithm used is unknown
	 * @since Envoy Common v0.2-alpha
	 */
	public LoginCredentials(String identifier, char[] password) throws NoSuchAlgorithmException {
		this.identifier		= identifier;
		this.id				= -1;
		passwordHash		= getSha256(toByteArray(password));
		this.registration	= true;
	}

	/**
	 * Creates an instance of {@link LoginCredentials} for an {@link User} who
	 * already registered himself (knows his id).
	 *
	 * @param id       the id of the user
	 * @param password the password of the user (will be converted to a hash)
	 * @throws NoSuchAlgorithmException if the algorithm used is unknown
	 * @since Envoy Common v0.2-alpha
	 */
	public LoginCredentials(long id, char[] password) throws NoSuchAlgorithmException {
		if (id <= 0) throw new IllegalArgumentException("Entered an illegal Id. Ids can not be below 1. Id was " + id);
		this.id				= id;
		passwordHash		= getSha256(toByteArray(password));
		this.registration	= false;
		this.identifier		= "";
	}

	private byte[] getSha256(byte[] input) throws NoSuchAlgorithmException { return MessageDigest.getInstance("SHA-256").digest(input); }

	private byte[] toByteArray(char[] chars) {
		byte[] bytes = new byte[chars.length * 2];
		for (int i = 0; i < chars.length; ++i) {
			bytes[i * 2]		= (byte) (chars[i] >> 8);
			bytes[i * 2 + 1]	= (byte) (chars[i]);
		}
		return bytes;
	}

	@Override
	public String toString() {
		try (Formatter form = new Formatter()) {
			if (registration) form.format("LoginCredentials[identifier=%s,passwordHash=", identifier);
			else form.format("LoginCredentials[id=%d,passwordHash=", id);
			for (byte element : passwordHash)
				form.format("%02x", element);
			return form.format(",registration=%b]", registration).toString();
		}
	}

	/**
	 * @return the identifier of the user performing the login
	 * @since Envoy Common v0.2-alpha
	 */
	public String getIdentifier() { return identifier; }

	/**
	 * @return the password hash of the user performing the login
	 * @since Envoy Common v0.2-alpha
	 */
	public byte[] getPasswordHash() { return passwordHash; }

	/**
	 * @return the id of the underlying user
	 * @since Envoy Common v0.2-alpha
	 */
	public long getId() { return id; }

	/**
	 * @return {@code true} if these credentials are used for user registration
	 *         instead of user login
	 * @since Envoy Common v0.2-alpha
	 */
	public boolean isRegistration() { return registration; }
}